@@ -88,15 +88,23 @@ def git_get_remote_branch_for_branch(branch):
     return branch
 
 
-def get_github_user_repo_from_url(url):
+def get_github_hostname_user_repo_from_url(url):
+    """Return hostname, user and repository to fork from.
+
+    :param url: The URL to parse
+    :return: hostname, user, repository
+    """
     parsed = parse.urlparse(url)
     if parsed.netloc == '':
         # Probably ssh
         host, sep, path = parsed.path.partition(":")
+        if "@" in host:
+            username, sep, host = host.partition("@")
     else:
         path = parsed.path[1:]
+        host = parsed.netloc
     user, repo = path.split("/", 1)
-    return user, repo[:-4] if repo.endswith('.git') else repo
+    return host, user, repo[:-4] if repo.endswith('.git') else repo
 
 
 def split_and_remove_empty_lines(s):
@@ -127,6 +135,12 @@ def git_get_log_titles(begin, end):
 
 
 def git_get_title_and_message(begin, end):
+    """Get title and message summary for patches between 2 commits.
+
+    :param begin: first commit to look at
+    :param end: last commit to look at
+    :return: number of commits, title, message
+    """
     titles = git_get_log_titles(begin, end)
 
     if len(titles) == 1:
@@ -135,13 +149,15 @@ def git_get_title_and_message(begin, end):
     else:
         title = "Pull request for " + end
         message = "\n".join(titles)
-    return (title, message)
+    return (len(titles), title, message)
 
 
 def git_pull_request(target_remote=None, target_branch=None,
                      title=None, message=None,
+                     comment=None,
                      comment_on_update=True,
-                     rebase=True):
+                     rebase=True,
+                     force_editor=False):
     branch = git_get_branch_name()
     if not branch:
         LOG.critical("Unable to find current branch")
@@ -177,23 +193,31 @@ def git_pull_request(target_remote=None, target_branch=None,
 
     LOG.debug("Remote URL for remote `%s' is `%s'", target_remote, target_url)
 
-    user_to_fork, reponame_to_fork = get_github_user_repo_from_url(target_url)
-    LOG.debug("GitHub user and repository to fork: %s/%s",
-              user_to_fork, reponame_to_fork)
+    hostname, user_to_fork, reponame_to_fork = (
+        get_github_hostname_user_repo_from_url(target_url)
+    )
+    LOG.debug("GitHub user and repository to fork: %s/%s on %s",
+              user_to_fork, reponame_to_fork, hostname)
 
     try:
-        user, password = get_login_password()
+        user, password = get_login_password(hostname)
     except KeyError:
         LOG.critical(
-            "Unable to find your GitHub credentials.\n"
+            "Unable to find your GitHub credentials for %s.\n"
             "Make sure you have a line like this in your ~/.netrc file:\n"
-            "machine github.com login <login> password <pwd>"
+            "machine %s login <login> password <pwd>",
+            hostname, hostname
         )
         return 35
 
     LOG.debug("Found GitHub user: `%s' password: <redacted>", user)
 
-    g = github.Github(user, password)
+    kwargs = {}
+    if hostname != "github.com":
+        kwargs['base_url'] = "https://" + hostname + "/api/v3"
+        LOG.debug("Using API base url `%s'", kwargs['base_url'])
+
+    g = github.Github(user, password, **kwargs)
     g_user = g.get_user()
     repo_to_fork = g.get_user(user_to_fork).get_repo(reponame_to_fork)
     repo_forked = g_user.create_fork(repo_to_fork)
@@ -251,38 +275,46 @@ def git_pull_request(target_remote=None, target_branch=None,
                 pull.edit(body=message)
                 LOG.debug("Updated pull-request message")
         if comment_on_update:
-            branch_sha = _run_shell_command(
-                ["git", "rev-parse", branch], output=True)
-            msg = "Pull-request updated, HEAD is now %s" % branch_sha
+            if not comment:
+                branch_sha = _run_shell_command(
+                    ["git", "rev-parse", branch], output=True)
+                comment = "Pull-request updated, HEAD is now %s" % branch_sha
             # FIXME(jd) we should be able to comment directly on a PR without
             # getting it as an issue but pygithub does not allow that yet
-            repo_to_fork.get_issue(pull.number).create_comment(msg)
-            LOG.debug("Commented: \"%s\"", msg)
+            repo_to_fork.get_issue(pull.number).create_comment(comment)
+            LOG.debug("Commented: \"%s\"", comment)
     else:
         # Create a pull request
-        editor = os.getenv("EDITOR")
-        if not editor:
-            LOG.warning(
-                "$EDITOR is unset, you will not be able to edit the "
-                "pull-request message")
-            editor = "cat"
-
-        git_title, git_message = git_get_title_and_message(
+        nb_of_commits, git_title, git_message = git_get_title_and_message(
             "%s/%s" % (target_remote, target_branch), branch)
 
-        title = title or git_title
-        message = message or git_message
+        # Do not run an editor if there's only one commit or if both title and
+        # message were specified
+        if (force_editor or
+           ((not title or not message) and nb_of_commits > 1) or
+           (((title and not message) or
+             (message and not title)) and nb_of_commits == 1)):
+            editor = os.getenv("EDITOR")
+            if not editor:
+                LOG.warning(
+                    "$EDITOR is unset, you will not be able to edit the "
+                    "pull-request message")
+                editor = "cat"
 
-        fd, bodyfilename = tempfile.mkstemp()
-        with open(bodyfilename, "w") as body:
-            body.write(title + "\n\n")
-            body.write(message + "\n")
-        os.system(editor + " " + bodyfilename)
-        with open(bodyfilename, "r") as body:
-            content = body.read().strip()
-        os.unlink(bodyfilename)
+            fd, bodyfilename = tempfile.mkstemp()
+            with open(bodyfilename, "w") as body:
+                body.write((title or git_title) + "\n\n")
+                body.write((message or git_message) + "\n")
+            os.system(editor + " " + bodyfilename)
+            with open(bodyfilename, "r") as body:
+                content = body.read().strip()
+            os.unlink(bodyfilename)
+
+            title, message = parse_pr_message(content)
+        else:
+            title = title or git_title
+            message = message or git_message
 
-        title, message = parse_pr_message(content)
         if title is None:
             LOG.critical("Pull-request message is empty, aborting")
             return 40
@@ -333,12 +365,21 @@ def main():
     parser.add_argument("--no-rebase", "-R",
                         action="store_true",
                         help="Don't rebase branch before pushing.")
+    parser.add_argument(
+        "--force-editor",
+        action="store_true",
+        default=False,
+        help="Force editor to run to edit pull-request message.")
     parser.add_argument(
         "--no-comment-on-update",
         action="store_true",
         default=False,
         help="Do not post a comment stating the pull-request has been updated."
     )
+    parser.add_argument(
+        "--comment", "-C",
+        help="Comment to publish when updating the pull-request"
+    )
 
     args = parser.parse_args()
 
@@ -357,7 +398,9 @@ def main():
         title=args.title,
         message=args.message,
         comment_on_update=not args.no_comment_on_update,
+        comment=args.comment,
         rebase=not args.no_rebase,
+        force_editor=args.force_editor,
     )
 
 
